@@ -59,6 +59,7 @@ private slots:
     void testStrokeWidth();
     void testMapViewBoxToTarget();
     void testRenderElement();
+    void testRenderElementToBounds();
     void constructorQXmlStreamReader() const;
     void loadQXmlStreamReader() const;
     void nestedQXmlStreamReader() const;
@@ -268,6 +269,15 @@ void tst_QSvgRenderer::testMapViewBoxToTarget()
         QCOMPARE(picture.boundingRect(), QRect(125, 125, 250, 250));
     }
 
+    { // Viewport and viewBox specified -> scale 500x500 square to 1000x750 while preserving aspect ratio gives 750x750
+        data = "<svg width=\"1000\" height=\"750\" viewBox=\"-250 -250 500 500\"><g><rect x=\"0\" y=\"0\" width=\"500\" height=\"500\" /></g></svg>";
+        QPicture picture;
+        QPainter painter(&picture);
+        QSvgRenderer rend(data);
+        rend.render(&painter);
+        painter.end();
+        QCOMPARE(picture.boundingRect(), QRect(500, 375, 750, 750));
+    }
 }
 
 void tst_QSvgRenderer::testRenderElement()
@@ -327,6 +337,41 @@ void tst_QSvgRenderer::testRenderElement()
 
 }
 
+void tst_QSvgRenderer::testRenderElementToBounds()
+{
+    // QTBUG-79933
+    QImage reference(400, 200, QImage::Format_ARGB32);
+    {
+        reference.fill(Qt::transparent);
+        QPainter p(&reference);
+        p.fillRect(  0,   0, 200, 100, Qt::blue);
+        p.fillRect(200, 100, 200, 100, Qt::blue);
+        p.fillRect(  0,   0, 100,  50, Qt::red);
+        p.fillRect(100,  50, 100,  50, Qt::red);
+        p.fillRect(200, 100, 100,  50, Qt::red);
+        p.fillRect(300, 150, 100,  50, Qt::red);
+    }
+
+    QImage rendering(400, 200, QImage::Format_ARGB32);
+    {
+        const char *const src =
+                "<svg viewBox=\"0 0 100 100\">" // Presence of a viewBox triggered QTBUG-79933
+                "<path id=\"el1\" d=\"M 80,10 H 0 V 0 h 40 v 20 h 40 z\" fill=\"red\" />"
+                "<path id=\"el2\" d=\"M 90,100 V 20 h 10 V 60 H 80 v 40 z\" fill=\"blue\" />"
+                "</svg>";
+        const QByteArray data(src);
+        QSvgRenderer rend(data);
+        rendering.fill(Qt::transparent);
+        QPainter p(&rendering);
+        rend.render(&p, "el1", QRectF(  0,   0, 200, 100));
+        rend.render(&p, "el2", QRectF(  0,   0, 200, 100));
+        rend.render(&p, "el1", QRectF(200, 100, 200, 100));
+        rend.render(&p, "el2", QRectF(200, 100, 200, 100));
+    }
+
+    QCOMPARE(reference, rendering);
+}
+
 void tst_QSvgRenderer::constructorQXmlStreamReader() const
 {
     const QByteArray data(src);
@@ -485,15 +530,15 @@ void tst_QSvgRenderer::matrixForElement() const
     QPainter painter(&image);
     QSvgRenderer renderer(data);
 
-    compareTransforms(QTransform(painter.worldMatrix()), QTransform(renderer.matrixForElement(QLatin1String("ichi"))));
+    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("ichi"))));
     painter.translate(-3, 1);
-    compareTransforms(QTransform(painter.worldMatrix()), QTransform(renderer.matrixForElement(QLatin1String("ni"))));
+    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("ni"))));
     painter.rotate(45);
-    compareTransforms(QTransform(painter.worldMatrix()), QTransform(renderer.matrixForElement(QLatin1String("san"))));
+    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("san"))));
     painter.scale(4, 2);
-    compareTransforms(QTransform(painter.worldMatrix()), QTransform(renderer.matrixForElement(QLatin1String("yon"))));
-    painter.setWorldMatrix(QMatrix(1, 2, 3, 4, 5, 6), true);
-    compareTransforms(QTransform(painter.worldMatrix()), QTransform(renderer.matrixForElement(QLatin1String("firkant"))));
+    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("yon"))));
+    painter.setWorldTransform(QTransform(1, 2, 3, 4, 5, 6), true);
+    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("firkant"))));
 }
 
 void tst_QSvgRenderer::boundsOnElement() const
@@ -1423,6 +1468,16 @@ void tst_QSvgRenderer::testUseElement()
         "   <circle fill=\"#a6ce39\" cx=\"0\" cy=\"0\" r=\"33\" />"
         "  </g>"
         " </defs>"
+        "</svg>",
+        // 17 - Indirect self referral
+        "<svg>"
+        " <defs>"
+        "   <g id=\"g0\">"
+        "     <g id=\"g1\"><use href=\"#g2\"/></g>"
+        "     <g id=\"g2\"><use href=\"#g1\"/></g>"
+        "   </g>"
+        " </defs>"
+        " <use xlink:href=\"#g0\" fill=\"black\"/>"
         "</svg>"
     };
 
